#ifndef BP_LIB_DISK_H
#define BP_LIB_DISK_H
#include <Uefi.h>
#include <Library/UefiLib.h>
#include <Library/MemoryAllocationLib.h>
#include <Protocol/BlockIo.h>
#include <Protocol/SimpleFileSystem.h>
#include <Guid/FileInfo.h>
typedef struct {
  UINT8 *content;
  UINTN file_size_bytes;
} bp_disk_FILE;
EFI_STATUS bp_disk_check_dir_existence(EFI_FILE_PROTOCOL *root,CHAR16 *dir);
EFI_STATUS bp_disk_check_file_existence(EFI_FILE_PROTOCOL *root,CHAR16 *file);
EFI_STATUS bp_disk_open_file(EFI_FILE_PROTOCOL *root,CHAR16 *file,EFI_FILE_PROTOCOL **opened_file);
EFI_STATUS bp_disk_get_file_info(EFI_FILE_PROTOCOL *opened_file,EFI_FILE_INFO **file_info);
EFI_STATUS bp_disk_read_file(EFI_FILE_PROTOCOL *opened_file,bp_disk_FILE **file);
#endif
